/*
---------------------------------------------------------------
SPDX-License-Identifier: BSD-3-Clause
 
Copyright (c) 2024, Renesas Electronics Corporation and/or its affiliates
 
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 
3. Neither the name of Renesas nor the names of its
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.
 
 
 
THIS SOFTWARE IS PROVIDED BY Renesas "AS IS" AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL RENESAS OR CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
---------------------------------------------------------------
 
This project writes a custom NDEF Text Record to an NDEF-formatted card.
Only UTF-8 encoded characters are supported as text.
Leading and trailing white spaces will be trimmed in the resulting text record.
Maximum text length is 249 characters.
Language code 'en' will be used for the resulting record.
Only NDEF-formatted cards are supported!
 
How to set up and run the example:
1. Prepare NDEF-formatted card(s).
2. Type the desired text into the Serial Monitor and press enter. Make sure "New Line" is enabled.
3. Place card on antenna and wait until card is written.
4. Remove card from antenna.
5. Repeate from step 2 as desired.
*/
 
#include <IotReader.h>
#include <Logging.h>
 
using namespace PtxIotReader;
 
bool status;
 
const uint8_t maxTextLength = 249U;
 
// Configure polling for type A, B, F and V types
// with 500 ms idle time between cycles
// Low Power Card Detection enabled with regular polling at every 10th cycle
// and stand-by mode enabled
const PollingConfig pollConfig = {.pollTypeA = 1U,
                                  .pollTypeB = 1U,
                                  .pollTypeF212 = 1U,
                                  .pollTypeV = 1U,
                                  .idleTime = 500U,
                                  .discoverMode = 10U,
                                  .enableStandBy = 1U};
 
void setup() {
  Serial.begin(115200);
  while (!Serial);  // wait for the port to be opened
  // initialize the reader instance
  status = IotReader::getReader().begin();
 
  if (!status) {
    Serial.println("ERROR: failed to initialize IoT reader");
    return;
  }
 
  Serial.println();
  Serial.println("WRITE NDEF TEXT RECORD TO CARD\n\n"
  "Only UTF-8 encoded characters are supported as text. Maximum length is 249 characters.\n"
  "Language code 'en' will be used for the resulting record.\n"
  "Only NDEF-formatted cards are supported!\n\n"
  "1. Type the desired text into the Serial Monitor and press enter. Make sure 'New Line' is enabled.\n"
  "2. Place card on antenna and wait until card is written.\n"
  "3. Remove card from antenna.\n"
  "4. Repeate as desired.");
  Serial.println();
 
  Serial.println("Type text to be written and press enter!");
  Serial.println();
}
 
void loop() {
  if (status) {
    // wait for serial input
    while (!Serial.available());
    if (Serial.available()) {
      String input = Serial.readStringUntil('\n');
      input.trim();
      Serial.print("Text received: ");
      Serial.println(input);
 
      if (input.length() > maxTextLength) {
        Serial.println("WARNING: input text length exceeds maximum limit, text will be clipped!");
      }
 
      // NDEF text record header: UTF-8 encoded, language code "en"
      // DO NOT change this vector, otherwise the NDEF message written to cards will be corrupted!
      std::vector<uint8_t> message = {0xD1, 0x01, 0x03, 0x54, 0x02, 0x65, 0x6E};
      uint8_t len = input.length() > maxTextLength ? maxTextLength : input.length();
      // update length in header
      message.at(2) += len;
      // insert text content
      message.insert(message.end(), input.begin(), input.begin() + len);
 
      Serial.print("Raw bytes to be written to card (0x): ");
      for (auto n : message) {
        Serial.print(n < 16 ? "0" : "");
        Serial.print(n, HEX);
        Serial.print(" ");
      }
      Serial.println();
 
      Serial.println(
        "Place a card on the antenna and do not remove it, until it "
        "is written!");
      // wait for card
      while(!IotReader::getReader().detectCard(pollConfig));
      Serial.println("Card found");
      printCardInfo(IotReader::getReader().getCardInfo());
      if (IotReader::getReader().ndefWrite(message)) {
        Serial.println("NDEF message written");
      } else {
        Serial.println("Could not write NDEF message");
        Serial.println("Make sure the card is NDEF-formatted");
      }
      IotReader::getReader().pollingStop();
      Serial.println();
      Serial.println("Type text to be written and press enter!");
      Serial.println();
      Serial.flush();
    }
    Serial.flush();
  }
}